//+------------------------------------------------------------------+
//|                                          EquidistantChannels.mq5 |
//|                                           Copyright 2016, denkir |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2016, denkir"
#property link      "https://www.mql5.com"
#property version   "1.00"
//--- include
#include "CFractalPoint.mqh"
//---
#property description "  ,"
#property description "   "
//---
#define BUFF_NUM 5
#property indicator_chart_window
#property indicator_buffers BUFF_NUM             //  
#property indicator_plots   5                    //  
//--- plot UpFractals
#property indicator_label1  "Up Fractals"        //  1
#property indicator_type1   DRAW_ARROW           //   1
#property indicator_color1  clrDodgerBlue        //   1
#property indicator_style1  STYLE_SOLID          //   1
#property indicator_width1  1                    //   1
//--- plot DnFractals
#property indicator_label2  "Down Fractals"      //  2
#property indicator_type2   DRAW_ARROW           //   2
#property indicator_color2  clrTomato            //   2
#property indicator_style2  STYLE_SOLID          //   2
#property indicator_width2  1                    //   2
//--- plot Upper Border
#property indicator_label3  "Upper Border"       //  3
#property indicator_type3   DRAW_LINE            //   3
#property indicator_color3  clrSteelBlue         //   3
#property indicator_width3  2                    //   3
//--- plot Lower Border
#property indicator_label4  "Lower Border"       //  4
#property indicator_type4   DRAW_LINE            //   4
#property indicator_color4  clrOrchid            //   4
#property indicator_width4  2                    //   4
//---
#property indicator_label5  "New channel"        //  5
#property indicator_type5   DRAW_NONE            //   5

//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
input int InpPrevFracNum=3;                        //  
input bool InpToPlotFracs=true;                    //  ?
input int InpBarsBeside=10;                        //  /  
input int InpBarsBetween=1;                        //  
input ENUM_RELEVANT_EXTREMUM InpRelevantPoint=RELEVANT_EXTREMUM_PREV; //  
sinput int InpLineWidth=3;                         //  
sinput bool InpToLog=true;                         //  ?
//+------------------------------------------------------------------+
//| Globals                                                          |
//+------------------------------------------------------------------+
double gUpFractalsBuffer[]; //   
double gDnFractalsBuffer[]; //   
double gUpperBuffer[];      //   
double gLowerBuffer[];      //   
double gNewChannelBuffer[]; //   
CFractalSet gFracSet;       //    
//---
int gMinRequiredBars;
int gLeftSide,gRightSide;
int gMaxSide;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int OnInit()
  {
//---  
   if(InpBarsBeside<1)
     {
      gLeftSide=gRightSide=2;

      PrintFormat("   \"-   \": %d.   : %d.",
                  InpBarsBeside,gLeftSide);
        } else {
      gLeftSide=gRightSide=InpBarsBeside;
     }
//---    
   gMinRequiredBars=gLeftSide+gRightSide+1;
   gMaxSide=gLeftSide;
//---  
   SetIndexBuffer(0,gUpFractalsBuffer,INDICATOR_DATA);
   SetIndexBuffer(1,gDnFractalsBuffer,INDICATOR_DATA);

//---   
   if(!InpToPlotFracs)
      for(int i=0;i<2;i++)
         PlotIndexSetInteger(i,PLOT_DRAW_TYPE,DRAW_NONE);
//---  
   PlotIndexSetInteger(0,PLOT_ARROW,217);
   PlotIndexSetInteger(1,PLOT_ARROW,218);
//--- c 
   PlotIndexSetInteger(0,PLOT_ARROW_SHIFT,-10);
   PlotIndexSetInteger(1,PLOT_ARROW_SHIFT,10);
//---  
   for(int i=0;i<2;i++)
     {
      PlotIndexSetInteger(i,PLOT_DRAW_BEGIN,gMinRequiredBars);
      PlotIndexSetDouble(i,PLOT_EMPTY_VALUE,0.0);
     }
   IndicatorSetInteger(INDICATOR_DIGITS,_Digits);
//---  
   SetIndexBuffer(2,gUpperBuffer,INDICATOR_DATA);
   SetIndexBuffer(3,gLowerBuffer,INDICATOR_DATA);
   SetIndexBuffer(4,gNewChannelBuffer,INDICATOR_DATA);
//---  
   for(int buff_idx=2;buff_idx<BUFF_NUM;buff_idx++)
      PlotIndexSetDouble(buff_idx,PLOT_EMPTY_VALUE,0.); 
//---
   return INIT_SUCCEEDED;
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//---       
   if(prev_calculated==0)
     {
      //---  
      ArrayInitialize(gUpFractalsBuffer,0.);
      ArrayInitialize(gDnFractalsBuffer,0.);
      ArrayInitialize(gUpperBuffer,0.);
      ArrayInitialize(gLowerBuffer,0.);
      ArrayInitialize(gNewChannelBuffer,0.);
     }
//---    [start]
   int startBar,lastBar;
//---
   if(rates_total<gMinRequiredBars)
     {
      Print("   ");
      return 0;
     }
//---
   if(prev_calculated<gMinRequiredBars)
      startBar=gLeftSide;
   else
      startBar=rates_total-gMinRequiredBars;
//---
   lastBar=rates_total-gRightSide;
   for(int bar_idx=startBar; bar_idx<lastBar && !IsStopped(); bar_idx++)
     {
      //---
      if(isUpFractal(bar_idx,gMaxSide,high))
         gUpFractalsBuffer[bar_idx]=high[bar_idx];
      else
         gUpFractalsBuffer[bar_idx]=0.0;
      //---
      if(isDnFractal(bar_idx,gMaxSide,low))
         gDnFractalsBuffer[bar_idx]=low[bar_idx];
      else
         gDnFractalsBuffer[bar_idx]=0.0;
     }
//---    [end]

//---     [start]
   if(prev_calculated>0)
     {
      //---    
      if(!gFracSet.IsInit())
         if(!gFracSet.Init(
            InpPrevFracNum,
            InpBarsBeside,
            InpBarsBetween,
            InpRelevantPoint,
            InpLineWidth,
            InpToLog
            ))
           {
            Print("   !");
            return 0;
           }
      //--- 
      gFracSet.Calculate(gUpFractalsBuffer,gDnFractalsBuffer,time,
                         gUpperBuffer,gLowerBuffer,
                         gNewChannelBuffer
                         );
     }
//---     [end]

//--- return value of prev_calculated for next call
   return rates_total;
  }
//+------------------------------------------------------------------+
//| Check if is Up Fractal function                                  |
//+------------------------------------------------------------------+
bool isUpFractal(int bar,int max,const double &High[])
  {
//---
   for(int i=1; i<=max; i++)
     {
      if(i<=gLeftSide && High[bar]<High[bar-i])
         return false;
      if(i<=gRightSide && High[bar]<=High[bar+i])
         return false;
     }
//---
   return true;
  }
//+------------------------------------------------------------------+
//| Check if is Down Fractal function                                |
//+------------------------------------------------------------------+
bool isDnFractal(int bar,int max,const double &Low[])
  {
//---
   for(int i=1; i<=max; i++)
     {
      if(i<=gLeftSide && Low[bar]>Low[bar-i])
         return false;
      if(i<=gRightSide && Low[bar]>=Low[bar+i])
         return false;
     }
//---
   return true;
  }
//+------------------------------------------------------------------+
